@@ -7,7 +7,7 @@ package Spreadsheet::WriteExcel::Formula;
 #
 # Used in conjunction with Spreadsheet::WriteExcel
 #
-# Copyright 2000-2007, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2010, John McNamara, jmcnamara@cpan.org
 #
 # Documentation after __END__
 #
@@ -24,7 +24,7 @@ use Carp;
 use vars qw($VERSION @ISA);
 @ISA = qw(Exporter);
 
-$VERSION = '2.20';
+$VERSION = '2.37';
 
 ###############################################################################
 #
@@ -58,6 +58,7 @@ sub new {
                     _ext_sheets     => {},
                     _ext_refs       => {},
                     _ext_ref_count  => 0,
+                    _ext_names      => {},
                  };
 
     bless $self, $class;
@@ -136,6 +137,7 @@ sub _init_parser {
                         | ref2d
                         | ref3d
                         | function
+                        | name
                         | '(' expr ')'  { [$item[2], 'ptgParen'] }
 
         # Match a string.
@@ -183,6 +185,10 @@ sub _init_parser {
                         | /[A-Z0-9-_.]+/ '(' list ')'
                         { ['_class', $item[1], $item[3], '_func', $item[1]] }
 
+        # Match a defined name.
+        name:           /[A-Za-z_]\w+/
+                        { ['_name', $item[1]] }
+
         # Boolean values.
         true:           'TRUE'  { [ 'ptgBool', 1 ] }
 
@@ -331,6 +337,14 @@ sub parse_tokens {
             $token      = shift @_;
             $parse_str .= $self->_convert_range3d($token, $class);
         }
+        elsif ($token =~ /^_name/) {
+            ($modifier  = $token) =~ s/_name//;
+            $class      = $class[-1];
+            $class      = 0 if $modifier eq 'R';
+            $class      = 1 if $modifier eq 'V';
+            $token = shift @_;
+            $parse_str .= $self->_convert_name($token, $class);
+        }
         elsif ($token eq '_func') {
             $token = shift @_;
             $parse_str .= $self->_convert_function($token, $num_args);
@@ -516,7 +530,16 @@ sub _convert_string {
 
     my $length = length($str);
 
-    # TODO string length
+    # Handle utf8 strings in perl 5.8.
+    if ($] >= 5.008) {
+        require Encode;
+
+        if (Encode::is_utf8($str)) {
+            $str = Encode::encode("UTF-16LE", $str);
+            $encoding = 1;
+        }
+    }
+
     die "String in formula has more than 255 chars\n" if $length > 255;
 
     return pack("CCC", $ptg{ptgStr}, $length, $encoding) . $str;
@@ -786,25 +809,10 @@ sub set_ext_sheets {
     my $worksheet   = shift;
     my $index       = shift;
 
-    #my $ref         = "$index:$index";
-
     # The _ext_sheets hash is used to translate between worksheet names
     # and their index
     $self->{_ext_sheets}->{$worksheet} = $index;
 
-
-    # 2D sheet refs such as '=Sheet1:Sheet2!A1' can only be added after all
-    # worksheets have been added.
-    #return 0 if $index < $self->{_ext_ref_count}; TODO
-
-
-    # The _ext_refs hash is used to correlate the external references used in
-    # formulas with the index stored in the Workbook EXTERNSHEET record.
-    #$self->{_ext_refs}->{$ref} = $index;
-    #$self->{_ext_ref_count}++;
-
-    # No errors
-    #return 1;
 }
 
 
@@ -812,26 +820,14 @@ sub set_ext_sheets {
 #
 # get_ext_sheets()
 #
-# This semi-public method is used to update the hash of sheet names. It is
-# updated by the add_worksheet() method of the Workbook class.
-#
-# TODO
+# This semi-public method is used to get the worksheet references that were
+# used in formulas for inclusion in the EXTERNSHEET Workbook record.
 #
 sub get_ext_sheets {
 
     my $self  = shift;
 
-    # TODO
-    my %refs = %{$self->{_ext_refs}};
-    return %refs;
-
-    #my @refs = sort {$refs{$a} <=> $refs{$b}} keys %refs;
-
-    #foreach my $ref (@refs) {
-    #    $ref = [split /:/, $ref];
-    #}
-
-    #return @refs;
+    return %{$self->{_ext_refs}};
 }
 
 
@@ -839,7 +835,7 @@ sub get_ext_sheets {
 #
 # get_ext_ref_count()
 #
-# TODO This semi-public method is used to update the hash of sheet names. It is
+# This semi-public method is used to update the hash of sheet names. It is
 # updated by the add_worksheet() method of the Workbook class.
 #
 sub get_ext_ref_count {
@@ -850,6 +846,43 @@ sub get_ext_ref_count {
 }
 
 
+###############################################################################
+#
+# _get_name_index()
+#
+# Look up the index that corresponds to an external defined name. The hash of
+# defined names is updated by the define_name() method in the Workbook class.
+#
+sub _get_name_index {
+
+    my $self        = shift;
+    my $name        = shift;
+
+    if (not exists $self->{_ext_names}->{$name}) {
+        die "Unknown defined name $name in formula\n";
+    }
+    else {
+        return $self->{_ext_names}->{$name};
+    }
+}
+
+
+###############################################################################
+#
+# set_ext_name()
+#
+# This semi-public method is used to update the hash of defined names.
+#
+sub set_ext_name {
+
+    my $self        = shift;
+    my $name        = shift;
+    my $index       = shift;
+
+    $self->{_ext_names}->{$name} = $index;
+}
+
+
 ###############################################################################
 #
 # _convert_function()
@@ -886,6 +919,38 @@ sub _convert_function {
 }
 
 
+###############################################################################
+#
+# _convert_name()
+#
+# Convert a symbolic name into a name reference.
+#
+sub _convert_name {
+
+    my $self     = shift;
+    my $name     = shift;
+    my $class    = shift;
+
+    my $ptgName;
+
+    my $name_index = $self->_get_name_index($name);
+
+    # The ptg value depends on the class of the ptg.
+    if    ($class == 0) {
+        $ptgName = $ptg{ptgName};
+    }
+    elsif ($class == 1) {
+        $ptgName = $ptg{ptgNameV};
+    }
+    elsif ($class == 2) {
+        $ptgName = $ptg{ptgNameA};
+    }
+
+
+    return pack 'CV', $ptgName, $name_index;
+}
+
+
 ###############################################################################
 #
 # _cell_to_rowcol($cell_ref)
@@ -1415,6 +1480,6 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
- MM-MMVII, John McNamara.
+ MM-MMX, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
